<?php
/**
 * @file mod_roomReservationRoomsManager.inc
 * Class for the management of rooms
 * @author Roland Hieber (roland.hieber@wilhelm-gym.net)
 * @date 28.12.2007
 * 
 * TODO: move into M Class Library
 * Copyright © 2007 Roland Hieber
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

require_once("db.inc");
require_once("mod_room-reservation/functions.inc");
require_once("mod_room-reservation/mod_roomReservationRoom.inc");
require_once("mod_room-reservation/mod_roomReservationConfig.inc");

/** Simple class for creating, editing and deleting rooms */
class mod_roomReservationRoomsManager {
  /** (mod_roomReservationConfig) Reference to the configuration */
  protected $oCfg;
  
  /**
   * Constructor
   * @param $oCfg (reference to mod_roomReservationConfig) Reference to the
   *  configuration object
   * @return rmRoomsManager
   */
  public function __construct(mod_roomReservationConfig &$oCfg) {
    $this->oCfg = $oCfg;
  }
  
  /**
   * Insert or update a room in the database
   * param $or (rsRoom) Room to store in the database
   * return (int) The UID of the booking, or <tt>-1</tt> in case of a failure.
   * Call getLastError() to get more information about the error.
   */
/**  public function write(rsRoom $or) {
    // Only administrators are allowed to create and alter rooms
    if(!$this->oCfg->userIsAdmin()) {
      // TODO throw exception
      setLastError(RS_ERROR_ACCESS_DENIED);
      return -1;
    }

    $strWhere = "";
    $strLog = "";

    // Update or insert?
    if($or->getUid() == null) {
      // No UID yet, insert new room
      $strLog = sprintf("Raum „%s“ angelegt", $or->getName());
    } else {
      $strWhere = "rsr_uid = ".qp(intval($or->getUid()));
      $strLog = sprintf("Raum „%s“ geändert", $or->getName());
    }
    
    $aPut["rsr_name"] = $or->getName();
    db_store("rooms", $aPut, $strWhere == "" ? null : $strWhere);
    
    $hQuery = db_query("SELECT currval('roomschedule_rooms_rsr_uid_seq') ".
      "FROM roomschedule_rooms;");
    $nNewUid = pg_fetch_result($hQuery, 0, "currval");
    
    rrInsertLog($strLog);
    
    // Return new UID
    return $nNewUid;
  }
  */
  /**
   * Delete a room from the database
   * param $nUid (int) Unique ID of the room
   * return (bool) <tt>true</tt> on success, otherwise <tt>false</tt>.
   *  Call getLastError() to get more information about the error.
   */
/*  public function delete($nUid) {
    // Only administrators are allowed to delete rooms
    if(!$this->oCfg->userIsAdmin()) {
      // TODO throw exception
      setLastError(RS_ERROR_ACCESS_DENIED);
      return false;
    }
    // Delete it from the database
    $strRoom = $this->getRoomName($nUid);
    if(!db_query("DELETE FROM roomschedule_rooms WHERE rsr_uid = $1;",
      intval($nUid))) {
        // TODO throw exception
      setLastError(RS_ERROR_SQL);
      return false;
    } else {
      rsInsertLog(sprintf("Raum „%s“ gelöscht", $strRoom));
      return true;
    }
  }
*/

  /**
   * Get a room by its OID. Returns <tt>null</tt> if the room was not found.
   * @param $nOid (integer) The OID of the room
   * @return mod_roomReservationRoom
   */
  static function getRoomByOid($nOid) {
    $o = null;
    $h = db_query("SELECT * FROM rooms WHERE oid = $1;", $nOid);
    if(pg_num_rows($h) > 0) {
      $arResult = pg_fetch_array($h);
      $o = new mod_roomReservationRoom($arResult["name"], 
        $arResult["room_no"], $arResult["floor"], $arResult["building"],
        $arResult["location"]);
    }
    return $o;
  }
  
  /**
   * Get a room by its name. Returns <tt>null</tt> if the room was not found.
   * @param $sName (string) The name of the room
   * @return mod_roomReservationRoom
   */
  static function getRoomByName($sName) {
    $o = null;
    $h = db_query("SELECT * FROM rooms WHERE name = $1;", $sName);
    if(pg_num_rows($h) > 0) {
      $arResult = pg_fetch_array($h);
      $o = new mod_roomReservationRoom($arResult["name"], 
        $arResult["room_no"], $arResult["floor"], $arResult["building"],
        $arResult["location"]);
    }
    return $o;
  }
  
  /**
   * Get all rooms from the database
   * @return array of mod_roomReservationRoom
   */
  static function getRooms() {
    $aoReturn = array();
    $hQuery = db_query("SELECT * FROM rooms ORDER BY name;");
    while($arResult = pg_fetch_array($hQuery)) {
      $aoReturn[] = new mod_roomReservationRoom($arResult["name"], 
        $arResult["room_no"], $arResult["floor"], $arResult["building"],
        $arResult["location"]);
    }
    return $aoReturn;
  }
}
?>
